
/* comp3: rpc.t -> client.ipp */

#include <map>
#include "test.h"
#include "data.hpp"

/* test */

static CLIENT * ifc = NULL;

void start_client (const string host)
{
   ifc = clnt_create (host.c_str (), TEST_PROGRAM, TEST_VERSION, "udp");
   if (ifc == NULL)
      clnt_pcreateerror (host.c_str ());
}

void stop_client ()
{
   if (ifc != NULL)
      clnt_destroy (ifc);
}

/* Group conversion */

static std::map < Group *, int > Group_dict;
static int Group_count = 0;

static int Group_store (Group * obj)
{
   assert (obj != NULL);
   assert (Group_dict.find (obj) == Group_dict.end ());
   Group_count ++;
   Group_dict [obj] = Group_count;
   return Group_count;
}

static int Group_conv (Group * obj)
{
   int result = 0;
   if (obj != NULL)
   {
      if (Group_dict.find (obj) != Group_dict.end ())
         result = Group_dict [obj];
      else
         result = Group_store (obj);
   }
   return result;
}

/* Section conversion */

static std::map < Section *, int > Section_dict;
static int Section_count = 0;

static int Section_store (Section * obj)
{
   assert (obj != NULL);
   assert (Section_dict.find (obj) == Section_dict.end ());
   Section_count ++;
   Section_dict [obj] = Section_count;
   return Section_count;
}

static int Section_conv (Section * obj)
{
   int result = 0;
   if (obj != NULL)
   {
      if (Section_dict.find (obj) != Section_dict.end ())
         result = Section_dict [obj];
      else
         result = Section_store (obj);
   }
   return result;
}

/* Item conversion */

static std::map < Item *, int > Item_dict;
static int Item_count = 0;

static int Item_store (Item * obj)
{
   assert (obj != NULL);
   assert (Item_dict.find (obj) == Item_dict.end ());
   Item_count ++;
   Item_dict [obj] = Item_count;
   return Item_count;
}

static int Item_conv (Item * obj)
{
   int result = 0;
   if (obj != NULL)
   {
      if (Item_dict.find (obj) != Item_dict.end ())
         result = Item_dict [obj];
      else
         result = Item_store (obj);
   }
   return result;
}

/* Group functions */

void test_Group_new (Group * obj)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_group_new_4 (Group_conv (obj), ifc);
      assert (_response != NULL);
   }
}

void test_Group_delete (Group * obj)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_group_delete_4 (Group_conv (obj), ifc);
      assert (_response != NULL);
   }
}

void test_Group_setTitle (Group * obj, string p_value)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_group_settitle_4 (Group_conv (obj), const_cast <char *> (p_value.c_str ()), ifc);
      assert (_response != NULL);
   }
}

void test_Group_setNumber (Group * obj, int p_value)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_group_setnumber_4 (Group_conv (obj), p_value, ifc);
      assert (_response != NULL);
   }
}

void test_Group_insertFirstSection (Group * obj, Section * item)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_group_insertfirstsection_4 (Group_conv (obj), Section_conv (item), ifc);
      assert (_response != NULL);
   }
}

void test_Group_insertLastSection (Group * obj, Section * item)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_group_insertlastsection_4 (Group_conv (obj), Section_conv (item), ifc);
      assert (_response != NULL);
   }
}

/* Section functions */

void test_Section_new (Section * obj)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_section_new_4 (Section_conv (obj), ifc);
      assert (_response != NULL);
   }
}

void test_Section_delete (Section * obj)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_section_delete_4 (Section_conv (obj), ifc);
      assert (_response != NULL);
   }
}

void test_Section_setIdentifier (Section * obj, string p_value)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_section_setidentifier_4 (Section_conv (obj), const_cast <char *> (p_value.c_str ()), ifc);
      assert (_response != NULL);
   }
}

void test_Section_setNumber (Section * obj, int p_value)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_section_setnumber_4 (Section_conv (obj), p_value, ifc);
      assert (_response != NULL);
   }
}

void test_Section_insertFirstItem (Section * obj, Item * item)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_section_insertfirstitem_4 (Section_conv (obj), Item_conv (item), ifc);
      assert (_response != NULL);
   }
}

void test_Section_insertLastItem (Section * obj, Item * item)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_section_insertlastitem_4 (Section_conv (obj), Item_conv (item), ifc);
      assert (_response != NULL);
   }
}

void test_Section_insertPrevSection (Section * obj, Section * item)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_section_insertprevsection_4 (Section_conv (obj), Section_conv (item), ifc);
      assert (_response != NULL);
   }
}

void test_Section_insertNextSection (Section * obj, Section * item)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_section_insertnextsection_4 (Section_conv (obj), Section_conv (item), ifc);
      assert (_response != NULL);
   }
}

void test_Section_removeSection (Section * obj)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_section_removesection_4 (Section_conv (obj), ifc);
      assert (_response != NULL);
   }
}

/* Item functions */

void test_Item_new (Item * obj)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_item_new_4 (Item_conv (obj), ifc);
      assert (_response != NULL);
   }
}

void test_Item_delete (Item * obj)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_item_delete_4 (Item_conv (obj), ifc);
      assert (_response != NULL);
   }
}

void test_Item_setText (Item * obj, string p_value)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_item_settext_4 (Item_conv (obj), const_cast <char *> (p_value.c_str ()), ifc);
      assert (_response != NULL);
   }
}

void test_Item_setComment (Item * obj, string p_value)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_item_setcomment_4 (Item_conv (obj), const_cast <char *> (p_value.c_str ()), ifc);
      assert (_response != NULL);
   }
}

void test_Item_insertPrevItem (Item * obj, Item * item)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_item_insertprevitem_4 (Item_conv (obj), Item_conv (item), ifc);
      assert (_response != NULL);
   }
}

void test_Item_insertNextItem (Item * obj, Item * item)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_item_insertnextitem_4 (Item_conv (obj), Item_conv (item), ifc);
      assert (_response != NULL);
   }
}

void test_Item_removeItem (Item * obj)
{
   if (ifc != NULL)
   {
      void * _response;
      _response = test_item_removeitem_4 (Item_conv (obj), ifc);
      assert (_response != NULL);
   }
}
