
/* comp3: rpc.t -> data.ipp */

/* Group */

void Group::setTitle (string p_value)
{
   if (p_value != Title)
   {
      Title = p_value;
      test_Group_setTitle (this, p_value);
   }
}

void Group::setNumber (int p_value)
{
   if (p_value != Number)
   {
      Number = p_value;
      test_Group_setNumber (this, p_value);
   }
}

void Group::linkSection (Section * item, Section * bef, Section * aft)
{
   assert (this != NULL);
   assert (item != NULL);

   assert (item->prevSection == NULL);
   assert (item->nextSection == NULL);
   assert (item->up == NULL);

   item->prevSection = bef;
   item->nextSection = aft;
   item->up = this;

   if (bef == NULL)
      this->firstSection = item;
   else
      bef->nextSection = item;

   if (aft == NULL)
      this->lastSection = item;
   else
      aft->prevSection = item;
}

void Group::insertFirstSection (Section * item)
{
   assert (this != NULL);
   linkSection (item, NULL, firstSection);
   test_Group_insertFirstSection (this, item);
}

void Group::insertLastSection (Section * item)
{
   assert (this != NULL);
   linkSection (item, lastSection, NULL);
   test_Group_insertLastSection (this, item);
}

void Group::removeAllSections ()
{
   Section * p = firstSection;
   while (p != NULL)
   {
      Section * t = p;
      p = p->nextSection;
      t->removeSection ();
      delete t;
   }
   firstSection = NULL;
   lastSection = NULL;
}

Group::~Group ()
{
   removeAllSections ();
   test_Group_delete (this);
}

Group::Group () :
   Title (),
   Number (0),
   firstSection (NULL),
   lastSection (NULL)
{
   test_Group_new (this);
}

/* Section */

void Section::setIdentifier (string p_value)
{
   if (p_value != Identifier)
   {
      Identifier = p_value;
      test_Section_setIdentifier (this, p_value);
   }
}

void Section::setNumber (int p_value)
{
   if (p_value != Number)
   {
      Number = p_value;
      test_Section_setNumber (this, p_value);
   }
}

void Section::insertPrevSection (Section * item)
{
   assert (this != NULL);
   assert (up != NULL);
   up->linkSection (item, prevSection, this);
   test_Section_insertPrevSection (this, item);
}

void Section::insertNextSection (Section * item)
{
   assert (this != NULL);
   assert (up != NULL);
   up->linkSection (item, this, nextSection);
   test_Section_insertNextSection (this, item);
}

void Section::removeSection ()
{
   assert (this != NULL);
   assert (up != NULL);
   test_Section_removeSection (this);

   if (prevSection == NULL)
      up->firstSection = nextSection;
   else
      prevSection->nextSection = nextSection;

   if (nextSection == NULL)
      up->lastSection = prevSection;
   else
      nextSection->prevSection = prevSection;

   prevSection = NULL;
   nextSection = NULL;
   up = NULL;
}

void Section::linkItem (Item * item, Item * bef, Item * aft)
{
   assert (this != NULL);
   assert (item != NULL);

   assert (item->prevItem == NULL);
   assert (item->nextItem == NULL);
   assert (item->up == NULL);

   item->prevItem = bef;
   item->nextItem = aft;
   item->up = this;

   if (bef == NULL)
      this->firstItem = item;
   else
      bef->nextItem = item;

   if (aft == NULL)
      this->lastItem = item;
   else
      aft->prevItem = item;
}

void Section::insertFirstItem (Item * item)
{
   assert (this != NULL);
   linkItem (item, NULL, firstItem);
   test_Section_insertFirstItem (this, item);
}

void Section::insertLastItem (Item * item)
{
   assert (this != NULL);
   linkItem (item, lastItem, NULL);
   test_Section_insertLastItem (this, item);
}

void Section::removeAllItems ()
{
   Item * p = firstItem;
   while (p != NULL)
   {
      Item * t = p;
      p = p->nextItem;
      t->removeItem ();
      delete t;
   }
   firstItem = NULL;
   lastItem = NULL;
}

Section::~Section ()
{
   if (up != NULL)
      removeSection ();
   removeAllItems ();
   test_Section_delete (this);
}

Section::Section () :
   Identifier (),
   Number (0),
   prevSection (NULL),
   nextSection (NULL),
   up (NULL),
   firstItem (NULL),
   lastItem (NULL)
{
   test_Section_new (this);
}

/* Item */

void Item::setText (string p_value)
{
   if (p_value != Text)
   {
      Text = p_value;
      test_Item_setText (this, p_value);
   }
}

void Item::setComment (string p_value)
{
   if (p_value != Comment)
   {
      Comment = p_value;
      test_Item_setComment (this, p_value);
   }
}

void Item::insertPrevItem (Item * item)
{
   assert (this != NULL);
   assert (up != NULL);
   up->linkItem (item, prevItem, this);
   test_Item_insertPrevItem (this, item);
}

void Item::insertNextItem (Item * item)
{
   assert (this != NULL);
   assert (up != NULL);
   up->linkItem (item, this, nextItem);
   test_Item_insertNextItem (this, item);
}

void Item::removeItem ()
{
   assert (this != NULL);
   assert (up != NULL);
   test_Item_removeItem (this);

   if (prevItem == NULL)
      up->firstItem = nextItem;
   else
      prevItem->nextItem = nextItem;

   if (nextItem == NULL)
      up->lastItem = prevItem;
   else
      nextItem->prevItem = prevItem;

   prevItem = NULL;
   nextItem = NULL;
   up = NULL;
}

Item::~Item ()
{
   if (up != NULL)
      removeItem ();
   test_Item_delete (this);
}

Item::Item () :
   Text (),
   Comment (),
   prevItem (NULL),
   nextItem (NULL),
   up (NULL)
{
   test_Item_new (this);
}
