
/* comp3: rpc.t -> server.ipp */

#include <iostream>
#include <map>
#include <rpc/pmap_clnt.h>
#include "test.h"
#include "data.hpp"

/* test */

extern "C" void test_program_4 (svc_req * rqstp, register SVCXPRT * transp);

void start_server ()
{
   SVCXPRT * transp;

   pmap_unset (TEST_PROGRAM, TEST_VERSION);

   transp = svcudp_create (RPC_ANYSOCK);
   if (transp == NULL)
   {
      cerr << "cannot create udp service" << endl;
      return;
   }

   if (! svc_register (transp, TEST_PROGRAM, TEST_VERSION, test_program_4, IPPROTO_UDP))
   {
      cerr << "unable to register udp service";
      return;
   }

   transp = svctcp_create (RPC_ANYSOCK, 0, 0);
   if (transp == NULL)
   {
      cerr << "cannot create tcp service" << endl;
      return;
   }

   if (! svc_register (transp, TEST_PROGRAM, TEST_VERSION, test_program_4, IPPROTO_TCP))
   {
      cerr << "unable to register tcp service";
      return;
   }

   svc_run ();
}

/* Group conversion */

std::map < int, Group * > Group_svc_dict;

Group * Group_svc_conv (int inx)
{
   Group * result = NULL;
   if (inx != 0)
   {
      result = Group_svc_dict [inx];
      assert (result != NULL);
   }
   return result;
}

void Group_svc_store (int inx, Group * obj)
{
   assert (Group_svc_dict.find (inx) == Group_svc_dict.end ());
   assert (obj != NULL);
   Group_svc_dict [inx] = obj;
}

/* Section conversion */

std::map < int, Section * > Section_svc_dict;

Section * Section_svc_conv (int inx)
{
   Section * result = NULL;
   if (inx != 0)
   {
      result = Section_svc_dict [inx];
      assert (result != NULL);
   }
   return result;
}

void Section_svc_store (int inx, Section * obj)
{
   assert (Section_svc_dict.find (inx) == Section_svc_dict.end ());
   assert (obj != NULL);
   Section_svc_dict [inx] = obj;
}

/* Item conversion */

std::map < int, Item * > Item_svc_dict;

Item * Item_svc_conv (int inx)
{
   Item * result = NULL;
   if (inx != 0)
   {
      result = Item_svc_dict [inx];
      assert (result != NULL);
   }
   return result;
}

void Item_svc_store (int inx, Item * obj)
{
   assert (Item_svc_dict.find (inx) == Item_svc_dict.end ());
   assert (obj != NULL);
   Item_svc_dict [inx] = obj;
}

/* Group functions */

extern "C" void * test_group_new_4_svc (Group_ref self, svc_req * request)
{
   static char * result;
   Group * obj = new Group ();
   Group_svc_store (self, obj);
   cout << "Created new " << "Group" << endl;
   return static_cast <void *> (& result);
}

extern "C" void * test_group_delete_4_svc (Group_ref self, svc_req * request)
{
   static char * result;
   cout << "Deleted " << "Group" << endl;
   return static_cast <void *> (& result);
}

extern "C" void * test_group_settitle_4_svc (Group_ref self, string_t value, svc_req * request)
{
   static char * result;
   Group * obj = Group_svc_conv (self);
   assert (obj != NULL);
   obj->setTitle (value);
   cout << "Group" << '.' << "setTitle" << '(' << value << ')' << endl;
   return static_cast <void *> (& result);
}

extern "C" string_t * test_group_gettitle_4_svc (Group_ref self, svc_req * request)
{
   static string_t result;
   Group * obj = Group_svc_conv (self);
   assert (obj != NULL);
   result = const_cast <char *> (obj->getTitle ().c_str ());
   cout << "Group" << '.' << "getTitle" << '=' << result << endl;
   return & result;
}

extern "C" void * test_group_setnumber_4_svc (Group_ref self, int value, svc_req * request)
{
   static char * result;
   Group * obj = Group_svc_conv (self);
   assert (obj != NULL);
   obj->setNumber (value);
   cout << "Group" << '.' << "setNumber" << '(' << value << ')' << endl;
   return static_cast <void *> (& result);
}

extern "C" int * test_group_getnumber_4_svc (Group_ref self, svc_req * request)
{
   static int result;
   Group * obj = Group_svc_conv (self);
   assert (obj != NULL);
   result = obj->getNumber ();
   cout << "Group" << '.' << "getNumber" << '=' << result << endl;
   return & result;
}

extern "C" void * test_group_insertfirstsection_4_svc (Group_ref self, Section_ref param, svc_req * request)
{
   static char * result;
   Group * obj = Group_svc_conv (self);
   assert (obj != NULL);
   Section * param_obj = Section_svc_conv (param);
   obj->insertFirstSection (param_obj);
   cout << "Group" << '.' << "insertFirstSection" << "(...)" << endl;
   return static_cast <void *> (& result);
}

extern "C" void * test_group_insertlastsection_4_svc (Group_ref self, Section_ref param, svc_req * request)
{
   static char * result;
   Group * obj = Group_svc_conv (self);
   assert (obj != NULL);
   Section * param_obj = Section_svc_conv (param);
   obj->insertLastSection (param_obj);
   cout << "Group" << '.' << "insertLastSection" << "(...)" << endl;
   return static_cast <void *> (& result);
}

extern "C" void * test_section_insertprevsection_4_svc (Section_ref self, Section_ref param, svc_req * request)
{
   static char * result;
   Section * obj = Section_svc_conv (self);
   assert (obj != NULL);
   Section * param_obj = Section_svc_conv (param);
   obj->insertPrevSection (param_obj);
   return static_cast <void *> (& result);
}

extern "C" void * test_section_insertnextsection_4_svc (Section_ref self, Section_ref param, svc_req * request)
{
   static char * result;
   Section * obj = Section_svc_conv (self);
   assert (obj != NULL);
   Section * param_obj = Section_svc_conv (param);
   obj->insertNextSection (param_obj);
   return static_cast <void *> (& result);
}

extern "C" void * test_section_removesection_4_svc (Section_ref self, svc_req * request)
{
   static char * result;
   Section * obj = Section_svc_conv (self);
   assert (obj != NULL);
   obj->removeSection ();
   return static_cast <void *> (& result);
}

/* Section functions */

extern "C" void * test_section_new_4_svc (Section_ref self, svc_req * request)
{
   static char * result;
   Section * obj = new Section ();
   Section_svc_store (self, obj);
   cout << "Created new " << "Section" << endl;
   return static_cast <void *> (& result);
}

extern "C" void * test_section_delete_4_svc (Section_ref self, svc_req * request)
{
   static char * result;
   cout << "Deleted " << "Section" << endl;
   return static_cast <void *> (& result);
}

extern "C" void * test_section_setidentifier_4_svc (Section_ref self, string_t value, svc_req * request)
{
   static char * result;
   Section * obj = Section_svc_conv (self);
   assert (obj != NULL);
   obj->setIdentifier (value);
   cout << "Section" << '.' << "setIdentifier" << '(' << value << ')' << endl;
   return static_cast <void *> (& result);
}

extern "C" string_t * test_section_getidentifier_4_svc (Section_ref self, svc_req * request)
{
   static string_t result;
   Section * obj = Section_svc_conv (self);
   assert (obj != NULL);
   result = const_cast <char *> (obj->getIdentifier ().c_str ());
   cout << "Section" << '.' << "getIdentifier" << '=' << result << endl;
   return & result;
}

extern "C" void * test_section_setnumber_4_svc (Section_ref self, int value, svc_req * request)
{
   static char * result;
   Section * obj = Section_svc_conv (self);
   assert (obj != NULL);
   obj->setNumber (value);
   cout << "Section" << '.' << "setNumber" << '(' << value << ')' << endl;
   return static_cast <void *> (& result);
}

extern "C" int * test_section_getnumber_4_svc (Section_ref self, svc_req * request)
{
   static int result;
   Section * obj = Section_svc_conv (self);
   assert (obj != NULL);
   result = obj->getNumber ();
   cout << "Section" << '.' << "getNumber" << '=' << result << endl;
   return & result;
}

extern "C" void * test_section_insertfirstitem_4_svc (Section_ref self, Item_ref param, svc_req * request)
{
   static char * result;
   Section * obj = Section_svc_conv (self);
   assert (obj != NULL);
   Item * param_obj = Item_svc_conv (param);
   obj->insertFirstItem (param_obj);
   cout << "Section" << '.' << "insertFirstItem" << "(...)" << endl;
   return static_cast <void *> (& result);
}

extern "C" void * test_section_insertlastitem_4_svc (Section_ref self, Item_ref param, svc_req * request)
{
   static char * result;
   Section * obj = Section_svc_conv (self);
   assert (obj != NULL);
   Item * param_obj = Item_svc_conv (param);
   obj->insertLastItem (param_obj);
   cout << "Section" << '.' << "insertLastItem" << "(...)" << endl;
   return static_cast <void *> (& result);
}

extern "C" void * test_item_insertprevitem_4_svc (Item_ref self, Item_ref param, svc_req * request)
{
   static char * result;
   Item * obj = Item_svc_conv (self);
   assert (obj != NULL);
   Item * param_obj = Item_svc_conv (param);
   obj->insertPrevItem (param_obj);
   return static_cast <void *> (& result);
}

extern "C" void * test_item_insertnextitem_4_svc (Item_ref self, Item_ref param, svc_req * request)
{
   static char * result;
   Item * obj = Item_svc_conv (self);
   assert (obj != NULL);
   Item * param_obj = Item_svc_conv (param);
   obj->insertNextItem (param_obj);
   return static_cast <void *> (& result);
}

extern "C" void * test_item_removeitem_4_svc (Item_ref self, svc_req * request)
{
   static char * result;
   Item * obj = Item_svc_conv (self);
   assert (obj != NULL);
   obj->removeItem ();
   return static_cast <void *> (& result);
}

/* Item functions */

extern "C" void * test_item_new_4_svc (Item_ref self, svc_req * request)
{
   static char * result;
   Item * obj = new Item ();
   Item_svc_store (self, obj);
   cout << "Created new " << "Item" << endl;
   return static_cast <void *> (& result);
}

extern "C" void * test_item_delete_4_svc (Item_ref self, svc_req * request)
{
   static char * result;
   cout << "Deleted " << "Item" << endl;
   return static_cast <void *> (& result);
}

extern "C" void * test_item_settext_4_svc (Item_ref self, string_t value, svc_req * request)
{
   static char * result;
   Item * obj = Item_svc_conv (self);
   assert (obj != NULL);
   obj->setText (value);
   cout << "Item" << '.' << "setText" << '(' << value << ')' << endl;
   return static_cast <void *> (& result);
}

extern "C" string_t * test_item_gettext_4_svc (Item_ref self, svc_req * request)
{
   static string_t result;
   Item * obj = Item_svc_conv (self);
   assert (obj != NULL);
   result = const_cast <char *> (obj->getText ().c_str ());
   cout << "Item" << '.' << "getText" << '=' << result << endl;
   return & result;
}

extern "C" void * test_item_setcomment_4_svc (Item_ref self, string_t value, svc_req * request)
{
   static char * result;
   Item * obj = Item_svc_conv (self);
   assert (obj != NULL);
   obj->setComment (value);
   cout << "Item" << '.' << "setComment" << '(' << value << ')' << endl;
   return static_cast <void *> (& result);
}

extern "C" string_t * test_item_getcomment_4_svc (Item_ref self, svc_req * request)
{
   static string_t result;
   Item * obj = Item_svc_conv (self);
   assert (obj != NULL);
   result = const_cast <char *> (obj->getComment ().c_str ());
   cout << "Item" << '.' << "getComment" << '=' << result << endl;
   return & result;
}

/* Group empty client functions */

void test_Group_new (Group * obj)
{ }

void test_Group_delete (Group * obj)
{ }

void test_Group_setTitle (Group * obj, string p_value)
{ }

void test_Group_setNumber (Group * obj, int p_value)
{ }

void test_Group_insertFirstSection (Group * obj, Section * item)
{ }

void test_Group_insertLastSection (Group * obj, Section * item)
{ }

/* Section empty client functions */

void test_Section_new (Section * obj)
{ }

void test_Section_delete (Section * obj)
{ }

void test_Section_setIdentifier (Section * obj, string p_value)
{ }

void test_Section_setNumber (Section * obj, int p_value)
{ }

void test_Section_insertFirstItem (Section * obj, Item * item)
{ }

void test_Section_insertLastItem (Section * obj, Item * item)
{ }

void test_Section_insertPrevSection (Section * obj, Section * item)
{ }

void test_Section_insertNextSection (Section * obj, Section * item)
{ }

void test_Section_removeSection (Section * obj)
{ }

/* Item empty client functions */

void test_Item_new (Item * obj)
{ }

void test_Item_delete (Item * obj)
{ }

void test_Item_setText (Item * obj, string p_value)
{ }

void test_Item_setComment (Item * obj, string p_value)
{ }

void test_Item_insertPrevItem (Item * obj, Item * item)
{ }

void test_Item_insertNextItem (Item * obj, Item * item)
{ }

void test_Item_removeItem (Item * obj)
{ }
