
/* str.cc */

#include "str.h"
// #include "conv.h"

OPEN_NAMESPACE

/* ---------------------------------------------------------------------- */

char * StrToChars (const string s)
{
   int len = s.length ();
   char * p = static_cast <char *> (malloc (len + 1));
   memcpy (p, s.c_str (), len);
   p [len] = 0;
   return p;
}

string CharsToStr (const char * c)
{
   if (c == NULL)
      return "";
   else
      return string (c);
}

/* ---------------------------------------------------------------------- */

string Head (const string s, int i)
{
   // first i characters
   return s.substr (0, i);
}

string Tail (const string s, int i)
{
   // skip first i characters
   return s.substr (i);
}

/* ---------------------------------------------------------------------- */

string UpperCase (string s)
{
   int len = s.length ();
   for (int i = 0; i<len; i ++)
   {
      char c = s[i];
      if (c >= 'a' && c <= 'z')
      {
         c = c - 'a' + 'A';
         s[i] = c;
      }
   }
   return s;
}

string LowerCase (string s)
{
   int len = s.length ();
   for (int i = 0; i<len; i ++)
   {
      char c = s[i];
      if (c >= 'A' && c <= 'Z')
      {
         c = c - 'A' + 'a';
         s[i] = c;
      }
   }
   return s;
}

string FirstUpperCase (string s)
{
   if (s.length () != 0)
      s[0] = UpCase (s[0]);
   return s;
}

string FirstLowerCase (string s)
{
   if (s.length () != 0)
      s[0] = LoCase (s[0]);
   return s;
}

/* ---------------------------------------------------------------------- */

bool IsValidIdent (const string value)
{
   bool result = false;
   int len = value.length ();

   if (len != 0)
   {
      result = IsLetter (value [0]);

      for (int i = 1; i<len && result; i++)
         if (! IsLetterOrDigit (value [i]))
            result = false;
   }

   return result;
}

bool IsValidNumber (const string value)
{
   int len = value.length ();

   bool result = (len > 0);
   for (int i = 0; i < len && result; i++)
      if (! IsDigit (value [i]))
         result = false;

   return result;
}

/* ---------------------------------------------------------------------- */

inline bool is_WhiteSpace (int n)
{
   return n == ' ' ||
          n == '\n' ||
          n == '\r' ||
          n == '\x09';
}

string Trim (string s)
{
   int len = s.length ();
   while (len > 0 && is_WhiteSpace (s[len-1]))
      len --;

   int ofs = 1;
   while (ofs <= len && is_WhiteSpace (s[ofs-1]))
      ofs ++;

   return s.substr (ofs-1, 1+len-ofs);
}

/* ---------------------------------------------------------------------- */

bool HasPrefix (const string value, char prefix)
{
   int len = value.length ();
   return len > 1 &&  value [0] == prefix;
}

string RemovePrefix (const string value, char prefix)
{
   int len = value.length ();
   if (len > 1 && value [0] == prefix)
      return value.substr (1);
   else
      return value;
}

bool HasPrefix (const string value, const string prefix)
{
   int len = value.length ();
   int pref_len = prefix.length ();

   return len > pref_len &&
          value.substr (0, pref_len) == prefix;
}

string RemovePrefix (const string value, const string prefix)
{
   int len = value.length ();
   int pref_len = prefix.length ();

   if (len > pref_len &&
       value.substr (0, pref_len) == prefix)

      return value.substr (pref_len);
   else
      return value;
}

/* ---------------------------------------------------------------------- */

bool HasWordPrefix (const string value, char prefix)
{
   int len = value.length ();

   return len > 1 && value [0] == prefix && IsUpperCaseLetter (value [1]);
}

string RemoveWordPrefix (const string value, char prefix)
{
   int len = value.length ();
   if (len > 1 && value [0] == prefix && IsUpperCaseLetter (value [1]))
      return value.substr (1);
   else
      return value;
}

bool HasWordPrefix (const string value, const string prefix)
{
   int len = value.length ();
   int pref_len = prefix.length ();

   return len > pref_len &&
          value.substr (0, pref_len) == prefix &&
          IsUpperCaseLetter (value [pref_len]);
}

string RemoveWordPrefix (const string value, const string prefix)
{
   int len = value.length ();
   int pref_len = prefix.length ();

   if (len > pref_len &&
       value.substr (0, pref_len) == prefix &&
       IsUpperCaseLetter (value [pref_len]))

      return value.substr (pref_len);
   else
      return value;
}

/* ---------------------------------------------------------------------- */

bool HasSuffix (const string value, char suffix)
{
   int len = value.length ();
   return len > 1 && value [len-1] == suffix;
}

string RemoveSuffix (const string value, char suffix)
{
   int len = value.length ();
   if (len > 1 && value [len-1] == suffix)
      return value.substr (0, len-1);
   else
      return value;
}

bool HasSuffix (const string value, const string suffix)
{
   int len = value.length ();
   int suffix_len = suffix.length ();

   return len > suffix_len &&
          value.substr (len-suffix_len) == suffix;
}

string RemoveSuffix (const string value, const string suffix)
{
   if (HasSuffix (value, suffix))
      return value.substr (0, value.length () - suffix.length ());
   else
      return value;
}

/* ---------------------------------------------------------------------- */

string ReplaceChars (string s, char c1, char c2)
{
   int len = s.length ();
   for (int i = 0; i < len; i++)
      if (s[i] == c1)
         s[i] = c2;
   return s;
}

string RemoveChars (const string s, char c)
{
   string result = "";
   int len = s.length ();

   for (int i = 0; i<len; i ++)
   {
      if (s[i] != c)
         result += s[i];
   }

   return result;
}

/* ---------------------------------------------------------------------- */

string ReplaceSpaces (const string s)
{
   return ReplaceChars (s, ' ', '_');
}

string RemoveSpaces (const string s)
{
   return RemoveChars (s, ' ');
}

/* ---------------------------------------------------------------------- */

string AddUnderlines (const string s)
{
   // add '_' between lower case and upper case letters

   string result = "";

   bool skip = true;
   int len = s.length ();

   for (int i = 0; i<len; i ++)
   {
      char c = s[i];

      if (c >= 'A' && c <= 'Z')
      {
         if (! skip)
            result += '_';
         c = c - 'A' + 'a'; // to lower case
         skip = true;
      }
      else
         skip = false;

      result += c;
   }

   return result;
}

string RemoveUnderlines (const string s)
{
   // remove '_' and convert next character to upper case
   string result = "";

   int len = s.length ();
   bool upc = false;

   for (int i = 0; i < len; i ++)
   {
      char c = s[i];

      if (c == '_')
         upc = true;
      else
      {
         if (upc && c >= 'a' && c <= 'z')
            c = c - 'a' + 'A'; // to upper case

         result = result + c;
         upc = false;
      }
   }

   return result;
}

/* ---------------------------------------------------------------------- */

string ReplaceTwoColonsWithUnderline (const string s)
{
   string result = "";
   int len = s.length ();

   if (len > 0)
   {
      char last_char = s[0];

      for (int i = 1; i<len; i ++)
      {
         char c = s[i];

         if (last_char == ':' && c == ':')
            last_char = '_';
         else
         {
            result = result + last_char;
            last_char = c;
         }
      }

      result = result + last_char;
   }
   return result;
}

/* ---------------------------------------------------------------------- */

string JoinIdentifiers (string a, string b)
{
   // join identifiers with '_'
   // or convert first chatacter of second parameter to upper case

   if (a != "" && b != "") // one empty => nothing to do
   {
      char aa = a [a.length () - 1]; // last character from a
      char bb = b[0]; // first character from b

      // any underline => join with underline
      if (a.find ('_') != string::npos || b.find ('_') != string::npos)
      {
         if (aa != '_' && bb != '_') // do not repeat underlines
            a = a + '_';
      }
      else
      {
         // convert to uppercase
         if (bb >= 'a' && bb <='z')
            b[0] = UpCase (bb);
      }
   }

   return a + b;
}

string AddNamespace (string a, string b)
{
   if (a == "")
      return b;
   else if (a == "::")
      return "::" + b;
   else
      return RemoveSuffix (a, "::") + "::" + b;
   // RemoveSuffix recognize suffix only when first parameter is longer then second
}

string RemoveAsterisk (string value)
{
   int len = value.length ();

   while (len > 0 && value [len-1] == ' ') len --; // skip spaces

   if (len > 0 && value [len-1] == '*')
   {
      len --; // skip asterisk
      while (len > 0 && value [len-1] == ' ') len --;  // skip spaces
   }

   return Head (value, len);
}

/* ---------------------------------------------------------------------- */

string RemovePath (string s)
{
   string::size_type inx = s.rfind ('/');

   if (inx != string::npos)
      s = s.substr (inx+1);

   return s;
}

string AddSlash (string s)
{
   int len = s.length ();
   if (len > 0 && s[len-1] != '/')
      return s + '/';
   else
      return s;
}

string RemoveLastSlash (string s)
{
   int len = s.length ();
   if (len > 1 && s[len-1] == '/')
      return s.substr (0, len-1);
   else
      return s;
}

string ModifyIdentifier (string s)
{
   int len = s.length ();

   if (len == 0)
      s = "_";
   else
   {
      if (! IsLetter (s[0]))
         s[0] = '_';

      for (int i = 1; i < len; i++)
         if (! IsLetterOrDigit (s[i]))
            s[i] = '_';
   }

   return s;
}

/*
string ModifyFileName (string s)
{
   int len = s.length ();

   if (len == 0)
      s = "_"; // empty string -> underline
   else
   {
      if (! IsLetterOrDigit (s[0]))
         s[0] = '_'; // first character -> underline

      for (int i = 1; i < len; i++)
         if (! IsLetterOrDigit (s[i]) || s[i] == '_')
            s[i] = '-'; // next characters -> dash
   }

   return s;
}
*/

string HeaderIdentifier (const string prefix, const string file_name)
{
   return "__" + UpperCase (ModifyIdentifier (prefix + "_" + RemovePath (file_name))) + "__";
}

/* ---------------------------------------------------------------------- */

string QuoteStrContent (const string value)
{
   string result = "";
   int len = value.length ();
   const char hex [16+1] = "0123456789abcdef";

   for (int i = 0; i < len; i++)
   {
      unsigned char ch = value [i];

      if (ch < ' ' || ch == 128 || ch >= 255)
      {
        result = result + '\\';
        switch (ch)
        {
           case '\a': result = result + 'a'; break;
           case '\b': result = result + 'b'; break;
           case '\f': result = result + 'f'; break;
           case '\n': result = result + 'n'; break;
           case '\r': result = result + 'r'; break;
           case '\t': result = result + 't'; break;
           case '\v': result = result + 'v'; break;

           default:
              result = result + 'x' + hex [ch >> 4] + hex [ch & 15];
              break;
        }
      }
      else
      {
         if (ch == '\\' || ch == '"')
            result = result + '\\';
         result = result + value [i]; // variable ch is unsigned char
      }
   }

   return result;
}

string QuoteStr (const string value, char quote)
{
   return quote + QuoteStrContent (value) + quote;
}

/* ---------------------------------------------------------------------- */

string RemoveQuotes (const string value, char quote)
{
   string result = value;

   int len = value.length ();
   if (len >= 2 && value [0] == quote && value [len-1] == quote)
   {
      result = "";
      int inx = 0;
      if (value [0] == quote) inx = 1; /* skip first quote */
      int max = len-1;
      if (value [len-1] == quote) max = len-2; /* skip last quote */

      while (inx <= max)
      {
         char ch = value [inx];

         if (ch != backslash || inx == max) /* simple character */
         {
            result += ch;
            inx ++;
         }
         else                               /* escape sequence */
         {
            inx ++;                         /* skip backslash */
            ch = value [inx];

            if (IsOctal (ch))               /* octal */
            {
               int n = 0;
               int cnt = 1;
               while (IsOctal (ch) && cnt <= 3 && inx <= max)
               {
                  n = n * 8 + ch - '0';
                  cnt ++;
                  inx ++;
                  ch = value [inx];
               }
               result += char (n);
            }

            else if (ch =='x' || ch =='X')  /* hex */
            {
               inx ++;
               int n = 0;
               while (IsHexDigit (ch) && inx <= max)
               {
                  int d;
                  if (ch >= 'A' && ch <= 'F')
                     d = ch - 'A' + 10;
                  else if (ch >= 'a' && ch <= 'f')
                     d = ch - 'a' + 10;
                  else
                     d = ch - '0';
                  n = n * 16 + d;
                  inx ++;
                  ch = value [inx];
               }
               result += char (n);
            }
            else
            {
               char n;
               switch (ch)                  /* other */
               {
                  case 'a': n = '\a'; break;
                  case 'b': n = '\b'; break;
                  case 'f': n = '\f'; break;
                  case 'n': n = '\n'; break;
                  case 'r': n = '\r'; break;
                  case 't': n = '\t'; break;
                  case 'v': n = '\v'; break;

                  case quote1:
                  case quote2:
                  case backslash:
                  case '?':
                  default:
                     n = ch; break;
               }

               result += n;
               inx ++;
               ch = value [inx];
            }
         }
      } /* end of loop */
   }

   return result;
}

CLOSE_NAMESPACE
